"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const AnnounceService_1 = require("../service/AnnounceService");
const CommandUtils_1 = require("../service/CommandUtils");
const RewardProgramService_1 = require("../service/RewardProgramService");
class EnrollRewardProgram extends command_1.Command {
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { flags } = this.parse(EnrollRewardProgram);
            CommandUtils_1.CommandUtils.showBanner();
            flags.password = yield CommandUtils_1.CommandUtils.resolvePassword(flags.password, flags.noPassword, CommandUtils_1.CommandUtils.passwordPromptDefaultMessage, true);
            yield new RewardProgramService_1.RewardProgramService(flags).enroll();
        });
    }
}
exports.default = EnrollRewardProgram;
EnrollRewardProgram.description = `It enrols the nodes in the rewards program by announcing the enroll transaction to the registration address.  You can also use this command to update the program registration when you change the agent keys (changing the agent-ca-csr) or server host.

Currently, the only program that can be enrolled post-launch is 'SuperNode'.`;
EnrollRewardProgram.examples = [
    `$ symbol-bootstrap enrollRewardProgram`,
    `$ symbol-bootstrap enrollRewardProgram --noPassword`,
    `$ symbol-bootstrap enrollRewardProgram --useKnownRestGateways`,
    `$ symbol-bootstrap enrollRewardProgram --password 1234 --url http://external-rest:3000`,
    `$ echo "$MY_ENV_VAR_PASSWORD" | symbol-bootstrap enrollRewardProgram --url http://external-rest:3000`,
];
EnrollRewardProgram.flags = Object.assign({ help: CommandUtils_1.CommandUtils.helpFlag, target: CommandUtils_1.CommandUtils.targetFlag }, AnnounceService_1.AnnounceService.flags);
//# sourceMappingURL=enrollRewardProgram.js.map