"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const fs_1 = require("fs");
const inquirer_1 = require("inquirer");
const path_1 = require("path");
const symbol_bootstrap_core_1 = require("symbol-bootstrap-core");
const service_1 = require("../service");
const clean_1 = __importDefault(require("./clean"));
const compose_1 = __importDefault(require("./compose"));
const config_1 = __importDefault(require("./config"));
class Pack extends command_1.Command {
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { flags } = this.parse(Pack);
            service_1.CommandUtils.showBanner();
            const logger = symbol_bootstrap_core_1.LoggerFactory.getLogger(symbol_bootstrap_core_1.LogType.Console);
            const preset = flags.preset;
            const assembly = flags.assembly;
            const targetZip = path_1.join(path_1.dirname(flags.target), `${preset}-${assembly || 'default'}-node.zip`);
            if (fs_1.existsSync(targetZip)) {
                throw new Error(`The target zip file ${targetZip} already exist. Do you want to delete it before repackaging your target folder?`);
            }
            logger.info('');
            logger.info('');
            if (!flags.ready &&
                !(yield inquirer_1.prompt([
                    {
                        name: 'offlineNow',
                        message: `Symbol Bootstrap is about to start working with sensitive information (certificates and voting file generation) so it is highly recommended that you disconnect from the network before continuing. Say YES if you are offline or if you don't care.`,
                        type: 'confirm',
                        default: true,
                    },
                ])).offlineNow) {
                logger.info('Come back when you are offline...');
                return;
            }
            flags.password = yield service_1.CommandUtils.resolvePassword(flags.password, flags.noPassword, service_1.CommandUtils.passwordPromptDefaultMessage, true);
            const service = yield new symbol_bootstrap_core_1.BootstrapService(this.config.root);
            const configOnlyCustomPresetFileName = 'config-only-custom-preset.yml';
            const configResult = yield service.config(Object.assign(Object.assign({}, flags), { privateKeyResolver: service_1.CommandUtils }));
            yield service.compose(flags, configResult.presetData);
            const noPrivateKeyTempFile = 'custom-preset-temp.temp';
            if (flags.customPreset) {
                yield symbol_bootstrap_core_1.BootstrapUtils.writeYaml(noPrivateKeyTempFile, symbol_bootstrap_core_1.CryptoUtils.removePrivateKeys(symbol_bootstrap_core_1.BootstrapUtils.loadYaml(flags.customPreset, flags.password)), flags.password);
            }
            else {
                yield symbol_bootstrap_core_1.BootstrapUtils.writeYaml(noPrivateKeyTempFile, {}, flags.password);
            }
            const zipItems = [
                {
                    from: flags.target,
                    to: 'target',
                    directory: true,
                },
                {
                    from: noPrivateKeyTempFile,
                    to: configOnlyCustomPresetFileName,
                    directory: false,
                },
            ];
            yield new symbol_bootstrap_core_1.ZipUtils(logger).zip(targetZip, zipItems);
            yield symbol_bootstrap_core_1.BootstrapUtils.deleteFile(noPrivateKeyTempFile);
            logger.info('');
            logger.info(`Zip file ${targetZip} has been created. You can unzip it in your node's machine and run:`);
            logger.info(`$ symbol-bootstrap start`);
        });
    }
}
exports.default = Pack;
Pack.description = 'It configures and packages your node into a zip file that can be uploaded to the final node machine.';
Pack.examples = [
    `$ symbol-bootstrap pack`,
    `$ symbol-bootstrap pack -c custom-preset.yml`,
    `$ symbol-bootstrap pack -p testnet -a dual -c custom-preset.yml`,
    `$ symbol-bootstrap pack -p mainnet -a dual --password 1234 -c custom-preset.yml`,
    `$ echo "$MY_ENV_VAR_PASSWORD" | symbol-bootstrap pack -p mainnet -a dual -c custom-preset.yml`,
];
Pack.flags = Object.assign(Object.assign(Object.assign(Object.assign({}, compose_1.default.flags), clean_1.default.flags), config_1.default.flags), { ready: command_1.flags.boolean({
        description: 'If --ready is provided, the command will not ask offline confirmation.',
    }) });
//# sourceMappingURL=pack.js.map