"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RewardProgramService = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const symbol_bootstrap_core_1 = require("symbol-bootstrap-core");
const symbol_sdk_1 = require("symbol-sdk");
const _1 = require(".");
const logger = symbol_bootstrap_core_1.LoggerFactory.getLogger(symbol_bootstrap_core_1.LogType.System);
class RewardProgramService {
    constructor(params) {
        this.params = params;
        this.configLoader = new symbol_bootstrap_core_1.ConfigLoader();
    }
    enroll(passedPresetData, passedAddresses) {
        return __awaiter(this, void 0, void 0, function* () {
            const presetData = passedPresetData !== null && passedPresetData !== void 0 ? passedPresetData : this.configLoader.loadExistingPresetData(this.params.target, this.params.password);
            const addresses = passedAddresses !== null && passedAddresses !== void 0 ? passedAddresses : this.configLoader.loadExistingAddresses(this.params.target, this.params.password);
            const customPreset = this.configLoader.loadCustomPreset(this.params.customPreset, this.params.password);
            if (!presetData.rewardProgramEnrollmentAddress) {
                logger.warn('This network does not have a reward program controller public key. Nodes cannot be registered.');
                return;
            }
            yield new _1.AnnounceService().announce(this.params.url, this.params.maxFee, this.params.useKnownRestGateways, this.params.ready, this.params.target, this.configLoader.mergePresets(presetData, customPreset), addresses, this, '1M+');
        });
    }
    createTransactions({ presetData, nodePreset, nodeAccount, deadline, maxFee, }) {
        return __awaiter(this, void 0, void 0, function* () {
            const transactions = [];
            const networkType = presetData.networkType;
            if (!nodePreset.rewardProgram) {
                logger.warn(`Node ${nodeAccount.name} hasn't been configured with rewardProgram: preset property.`);
                return transactions;
            }
            if (!presetData.rewardProgramEnrollmentAddress) {
                return transactions;
            }
            const rewardProgramEnrollmentAddress = symbol_sdk_1.Address.createFromRawAddress(presetData.rewardProgramEnrollmentAddress);
            const agentPublicKey = nodeAccount.transport.publicKey;
            if (!agentPublicKey) {
                logger.warn(`Cannot resolve harvester public key of node ${nodeAccount.name}`);
                return transactions;
            }
            if (!nodePreset.host) {
                logger.warn(`Node ${nodeAccount.name} public host name hasn't been provided! Please use 'host: myNodeHost' custom preset param.`);
                return transactions;
            }
            const agentUrl = nodePreset.agentUrl || `https://${nodePreset.host}:${nodePreset.rewardProgramAgentPort || presetData.rewardProgramAgentPort}`;
            const certFolder = symbol_bootstrap_core_1.BootstrapUtils.getTargetNodesFolder(this.params.target, false, nodePreset.name, 'agent');
            const base64AgentCaCsrFile = fs_1.readFileSync(path_1.join(certFolder, 'agent-ca.csr.pem'), 'base64');
            const plainMessage = `enroll ${agentUrl} ${base64AgentCaCsrFile}`;
            const message = symbol_sdk_1.PlainMessage.create(plainMessage);
            logger.info(`Creating enrolment transfer with message '${plainMessage}'`);
            const transaction = symbol_sdk_1.TransferTransaction.create(deadline, rewardProgramEnrollmentAddress, [], message, networkType, maxFee);
            return [transaction];
        });
    }
}
exports.RewardProgramService = RewardProgramService;
RewardProgramService.defaultParams = {
    useKnownRestGateways: false,
    target: symbol_bootstrap_core_1.BootstrapUtils.defaultTargetFolder,
    url: 'http://localhost:3000',
};
//# sourceMappingURL=RewardProgramService.js.map